@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2020-2021 Arm. All rights reserved.
+# Copyright (C) 2020-2022 Arm Limited or its affiliates and Contributors. All rights reserved.
 # SPDX-License-Identifier: Apache-2.0
 #
 """Utility script to abstract git operations for our CI scripts."""
@@ -161,7 +161,11 @@ class GitWrapper:
         Returns:
             corresponding branch
         """
-        return self.get_branch(configuration.get_value(ConfigurationVariable.MASTER_BRANCH))
+        main = configuration.get_value(ConfigurationVariable.MASTER_BRANCH)
+        branch = self.get_branch(main)
+        if branch:
+            return branch
+        return self.get_remote_branch(main)
 
     def get_beta_branch(self) -> Any:
         """Gets the `beta` branch.
@@ -169,7 +173,11 @@ class GitWrapper:
         Returns:
             corresponding branch
         """
-        return self.get_branch(configuration.get_value(ConfigurationVariable.BETA_BRANCH))
+        beta = configuration.get_value(ConfigurationVariable.BETA_BRANCH)
+        branch = self.get_branch(beta)
+        if branch:
+            return branch
+        return self.get_remote_branch(beta)
 
     def is_release_branch(self, branch_name: Optional[str]) -> bool:
         """Checks whether the branch is a `release` branch or not.
@@ -181,10 +189,8 @@ class GitWrapper:
             True if the branch is used for `release` code; False otherwise
         """
         branch_pattern = configuration.get_value(ConfigurationVariable.RELEASE_BRANCH_PATTERN)
-        if not branch_pattern or not branch_name:
-            return False
-        is_release: Optional[Any] = re.search(branch_pattern, str(branch_name))
-        return True if is_release else False
+        is_release, _ = self._is_branch_of_type(branch_name, branch_pattern)
+        return is_release
 
     def fetch(self) -> None:
         """Fetches latest changes."""
@@ -472,6 +478,28 @@ class GitWrapper:
         """
         self.repo.git.push(force=True, tags=True)
 
+    def is_dirty(self) -> bool:
+        """Determines whether repository is dirty.
+
+        Repository is considered dirty when git status returns elements which are not committed.
+        """
+        return self.repo.is_dirty(untracked_files=True)
+
+    def clean(self) -> None:
+        """Cleans the repository.
+
+        Performs a force clean.
+        """
+        if self.is_dirty():
+            self.repo.git.clean(force=True, x=True, d=True)
+
+    def stash(self) -> None:
+        """Stashes the repository.
+
+        Performs a stash.
+        """
+        self.repo.git.stash(all=True, quiet=True)
+
     def configure_for_github(self) -> None:
         """Reconfigures the repository.
 
@@ -521,11 +549,23 @@ class GitWrapper:
 
     def list_files_added_on_current_branch(self) -> List[str]:
         """Returns a list of files changed against master branch."""
-        master_branch_commit = self.repo.commit(configuration.get_value(ConfigurationVariable.MASTER_BRANCH))
+        master_branch = self.get_master_branch()
+        beta_branch = self.get_beta_branch()
+        master_branch_commit = self.repo.commit(master_branch)
+        beta_branch_commit = self.repo.commit(beta_branch)
         current_branch_commit = self.repo.commit(self.get_current_branch())
-        changes = self.get_changes_list(
-            self.get_branch_point(master_branch_commit, current_branch_commit), current_branch_commit, change_type="a"
-        )
+        # Finding the baseline branch to consider
+        master_branch_point = self.repo.commit(self.get_branch_point(master_branch_commit, current_branch_commit))
+        beta_branch_point = self.repo.commit(self.get_branch_point(beta_branch_commit, current_branch_commit))
+        branch_point = master_branch_point
+        if not master_branch:
+            branch_point = beta_branch_point
+        elif beta_branch and master_branch:
+            if beta_branch_point.committed_datetime > master_branch_point.committed_datetime:
+                # The branch point off `beta` is more recent than off `master`.
+                # Hence, the difference between current and `beta` should be considered.
+                branch_point = beta_branch_point
+        changes = self.get_changes_list(branch_point, current_branch_commit, change_type="a")
         return changes
 
     def is_current_branch_feature(self) -> bool:
@@ -536,6 +576,18 @@ class GitWrapper:
         is_release = self.is_release_branch(current_branch)
         return not (is_master or is_beta or is_release)
 
+    def is_current_branch_of_type(self, pattern: str) -> (bool, Optional[List[Any]]):
+        """Returns boolean indicating whether the current branch follows the pattern and the list of groups if any."""
+        return self._is_branch_of_type(self.get_current_branch(), pattern)
+
+    def _is_branch_of_type(self, branch_name: Optional[str], pattern: Optional[str]) -> (bool, Optional[List[Any]]):
+        if not pattern:
+            return False, None
+        if not branch_name:
+            return False, None
+        match = re.search(pattern, str(branch_name))
+        return True if match else False, match.groups() if match else None
+
     @property
     def uncommitted_changes(self) -> List[Path]:
         """Gets list of uncommitted files.
@@ -549,6 +601,19 @@ class GitWrapper:
 
         return [Path(self.root).joinpath(line.strip().split(" ")[-1]) for line in status.splitlines()]
 
+    @property
+    def uncommitted_staged_changes(self) -> List[Path]:
+        """Gets list of uncommitted staged changes.
+
+        Returns:
+             list of uncommitted staged changes
+        """
+        staged = self.repo.git.diff(staged=True, name_only=True)
+        if not staged:
+            return []
+
+        return [Path(self.root).joinpath(line.strip()) for line in staged.splitlines()]
+
     @staticmethod
     def _apply_modifications(destination: Path, modified_file: Path) -> None:
         logger.info(f"Applying change in {modified_file} to {destination}")
@@ -575,6 +640,25 @@ class GitWrapper:
                 GitWrapper._apply_modifications(destination, f)
             else:
                 GitWrapper._apply_deletions(destination)
+        for f in self.uncommitted_staged_changes:
+            other_repo.add(f.relative_to(self.root))
+
+    def get_corresponding_path(self, path_in_initial_repo: Path) -> Path:
+        """Gets the path in current repository corresponding to path in initial repository.
+
+        If current repository is not a clone, then identical absolute path is returned.
+
+        Args:
+            path_in_initial_repo: path to a file/directory in initial repository.
+
+        Returns:
+             corresponding path.
+        """
+        return (
+            path_in_initial_repo
+            if path_in_initial_repo.is_absolute()
+            else Path(self.root).joinpath(path_in_initial_repo)
+        )
 
 
 class ProjectGitWrapper(GitWrapper):
